function mufftres(varargin)
%
% usage MUFFTRES(LB)
% FFT of residues (data-fit)
% with hypergaussian window of width LB (in mus^-1).
% A satisfactory fit must be performed first, with MUFIT.
% Hint: choose LB close to the relaxation rate of the component
% you want to see.
%
% usage MUFFTRES(LB,N)
% residues are calculated subtracting only components n
% (e.g. in fit bamumu, MUFFTRES(LB,[1 3]) will subtract from the
% data the best fit, setting to zero the parameters of the second 
% component, mu) 
%
% usage MUFFTRES(LB,N,RANGEFFT)
% RANGEFFT is the range to be displayed (in MHz, e.g. [1 50])

    global MU_MODEL_F
    global MU_FIG
    global MU_ADD
    global MU_DATA
    global MU_GEAR
    
    rangefft=[];
    transparency=0.3;
    blue=[0.1 0.1 0.8];
    red=[0.8 0.1 0.1];
    mag=[0.6 0.1 0.6];
    if ~isfield(MU_MODEL_F,'CHI')
        disp('Fit not present')
    else
        n=1:length(MU_MODEL_F.COMPONENTNUMBER);
        switch length(varargin)
          case 0
            % autocall from Phase_CallBack, recover values
            lb=MU_FIG.FFTLB;
            n=MU_FIG.FFTN;
            rangefft=MU_FIG.FFTRANGE;
          case 1
            % never happening yet
            MU_FIG.FFTLB=varargin{1};
          case 2
            % never happening yet
            MU_FIG.FFTLB=varargin{1};
            n=varargin{2};
          case 3
            % call from MUFIT_GUI, store values
            lb=varargin{1};
            MU_FIG.FFTLB=lb;
            n=varargin{2};
            MU_FIG.FFTN=n;
            rangefft=varargin{3};
            MU_FIG.FFTRANGE=rangefft;
        end
        [mupout dum parname pf]=mupar(1,n);
        range=MU_MODEL_F.RANGE;
        t=MU_DATA.TIME(range(1):range(2)); % time
        y=MU_DATA.ASYMMETRY(range(1):range(2))-mufcn(mupout,t); % residues
        ey=MU_DATA.ASYMERROR(range(1):range(2));
        % extrapolate to nearly t=0
        
        dt=t(2)-t(1);
        nt0=round(t(1)/dt); % number of bins between 0 and t(1)
        dfr=1/t(end); % real frequency resolution
                      %t=[t(1)+dt*[-nt0:-1] t]; % time from t~0 to t(end)
        l=length(t);
        n=2*l;%^(round(log2(l))+1); % doubled range, not a power of 2
        % zero padding to n
        % t=[t t(l)+dt*[1:n-l]]; % extended time
        filter=exp(-(dt*[0:n-1]*lb).^3); % filter is applied as if
                                         % first good bin were t=0
        filter=filter/sum(filter)/dt; % approx normalization
        % try hypergauss n=3, varying exponent
        y=[y zeros(1,n-l)]; %  zero padded residues
        y=y.*filter; % filtered residue
        ey=[ey zeros(1,n-l)]; %  zero padded errore
        ey=ey.*filter; % filtered errors
        if ~isfield(MU_FIG,'FFTPHI')
            MU_FIG.FFTPHI=0; % no phase0 correction
            MU_FIG.FFTDPHIDW=0; % no phase1 correction
            % 2*pi*MU_GEAR.OFFSET*MU_DATA.RESNS/1e3;
            % start value based on Delta t between t=0 and first good bin
            % (offset)
        end
        a=fft(y); % amplitudes
        df=1/n/dt; % apparent frequency resolution
        f=[0:n/2 -n/2+1:-1]*df; %positive frequencies
        FMAX=max(f);
        % APPLY PHASE CORRECTION
        a=a.*exp(i*(-MU_FIG.FFTPHI*pi/180-(MU_FIG.FFTDPHIDW-t(1)*2* ...
                                          pi)*f));
        % a positive phase0 correction thus compensate a positiva
        % phase of a cosine wave
        % while a positive phase2 coefficient means that the
        % correct t0 is moved FORWARD with respect to nt0 and dt
        % (i.e. to the time value encoded in t(1), which is
        % therefore LARGER than it should). In this way the dead
        % time (first good bin-nt0) should be automatically compensated
        if ~isempty(rangefft) % translate freq range into bins
            is=find(f<=rangefft(1)); % index of frequencies <= rangefft(1)
            is=is(find(is<=n/2)); % positive ones
            start=max(is);
            stop=min(find(f>=rangefft(2))); % here it is fine: 
            % positive values come first
            stop=min([stop n/2+1]);
        else
            start=1;
            stop=find(f==max(f));
        end
        f=f(start:stop);
        ar=real(a(start:stop)); 
        ai=imag(a(start:stop)); 
% noise in the FFT: data, tn, yn with standard deviations eyn 
% FFT ampli am=sum_n yn exp(i wm tn) 
% => eam^2=sum_n |exp(i wm tn)|^2 eyn^2 = sum_n eyn^2 uniform
% ea is a distance, hence the radius of one standard deviation
% both ai and ar have the same sigma=ea
% hence ap has sigma=sqrt(2)*ea 
        ea=sqrt(sum(ey.^2))*ones(size(ai)); % noise level

        ap=ai.^2+ar.^2; % power
        % ab=sqrt(ab); % absolute value
        figure(MU_FIG.FFT)
        set(MU_FIG.FFT,'Name',MU_FIG.FFTNAME,'Visible','off',...
              'MenuBar','None');
        clf
        
        empty=[];
        Xmin=0.085;
        Ymin=0.1;
        Ywidth=0.3;
        Xwidth=0.8-Xmin;
        Ywidth=(0.92-Ymin)/3;
        hiamp=axes('position',[Xmin Ymin Xwidth Ywidth]);
        
        %        roff=max(ar)*1.3;
        
        x=[f f(end:-1:1)];
        hold on
        for k=1:length(f)
            heu=plot([f(k) f(k)],[ai(k) ai(k)+ea(k)],'b-','color',blue);
            hed=plot([f(k) f(k)],[ai(k) ai(k)-ea(k)],'b-','color',blue);
        end
        he=plot(f,ai,'bo','markerfacecolor','w',...
            'markeredgecolor',blue,'markersize',4);
        y=[ai zeros(size(ai))];
        fill(x,y,blue,'facealpha',transparency,'linestyle','none')
        hold off
        xy=axis;
        axis([min(f) max(f) [min(ai) max(ai)]*1.1])               
%        ytl=get(gca,'yticklabel');
%        yti = str2num(ytl(end))
%        dyti = yti - str2num(ytl(end-1))
%        if xy(4) - yti < dyti*0.4, ytl(end-1)=' '; ytl(end)=' '; end  
%        ylt=str2num(ytl{end});
%        dytl=ylt-str2num(ytl{end-1});
%        if xy(4)-ylt<dytl*0.4, ytl{end-1}=' '; ytl{end}=' '; end
%        set(gca,'yticklabel',ytl);
        xlabel('\nu (MHz)')
        ylabel('Imag Amp.')

        hramp=axes('position',[Xmin Ymin+Ywidth Xwidth Ywidth]);

        hold on
        for k=1:length(f)
            heu=plot([f(k) f(k)],[ar(k) ar(k)+ea(k)],'r-','color',red);
            hed=plot([f(k) f(k)],[ar(k) ar(k)-ea(k)],'r-','color',red);
        end
        he=plot(f,ar,'bo','markerfacecolor','w',...
            'markeredgecolor',red,'markersize',4);
        y=[ar zeros(size(ar))];
        fill(x,y,red,'facealpha',transparency,'linestyle','none')
        xy=axis;
        axis([min(f) max(f) [min(ar) max(ar)]*1.1])
        %ytl=get(gca,'yticklabel');
        %ylt=str2num(ytl{end});
        %dytl=ylt-str2num(ytl{end-1});
        %if xy(4)-ylt<dytl*0.4, ytl{end-1}=' '; ytl{end}=' '; end 
        %set(gca,'yticklabel',ytl);
        set(gca,'xticklabel',empty)%,'yaxislocation','right')
        ylabel('Real Amp.')
        hold off
        
        hpow=axes('position',[Xmin Ymin+2*Ywidth Xwidth Ywidth]);
        
        lw=2;
        da=0.2;
        eap=2*ea.*sqrt(ap);
        hold on
        for k=1:length(f)
            heu=plot([f(k) f(k)],[ap(k) ap(k)+eap(k)],'b-','color',mag);
            hed=plot([f(k) f(k)],[ap(k) ap(k)-eap(k)],'b-','color',mag);
        end
        he=plot(f,ap,'bo','markerfacecolor','w',...
            'markeredgecolor',mag,'markersize',4);
        y=[ap zeros(size(ap))];
        fill(x,y,mag,'facealpha',transparency,'linestyle','none')
        hold off
        set(gca,'xticklabel',empty)
        ylabel('Power (a.u.)')

        %size(f)
        %[min(f) max(f) 0 max(ap)*1.1]
        xy=axis;
        xy=[min(f) max(f) 0 max(ap)*1.1];
        axis(xy);
        dy=xy(4)-xy(3);
        ytext=xy(4)+0.06*dy;
        dtext=(xy(2)-xy(1))*0.02;
        if isempty(MU_ADD)
            runs=int2str(MU_DATA.NRUN);
        else
            l=length(MU_ADD);
            runs=[int2str(MU_DATA.NRUN) '+' int2str(MU_ADD)];
        end
        blue=[.2 .2 .8];
        fsiz=11;
        text(xy(1)-dtext,ytext,[runs '   ' MU_DATA.SAMPLE ...
                   MU_DATA.TEMPERATURE ' ' MU_DATA.FIELD ...
                    ' ' MU_DATA.ORIENTATION] ...
             ,'fontsize',fsiz,'fontweight','demi','color',blue)
        fsizlab=20;
        text(xy(2)+3*dtext,ytext+0.02*dy,'\mu lab',...
             'fontsize',fsizlab,'fontweight','demi',...
             'fontname','Arial','color',blue);
        ytext=xy(3)+0.82*dy;
        text(xy(2)+dtext,ytext,['LB=' num2str(lb) ' {\mu}s^{-1}'],...
              'fontsize',fsiz,'fontweight','demi','color',blue)
        ytext=xy(3)+0.66*dy;
        text(xy(2)+dtext,ytext,['\alpha=' num2str(MU_MODEL_F.ALPHA) ...
                   ],'fontsize',8);
        ytext=xy(3)+0.5*dy;
        text(xy(2)+dtext,ytext,['F:[' musgroup(1,0) ']'],'fontsize',8);
        ytext=xy(3)+0.38*dy;
        text(xy(2)+dtext,ytext,['B:[' musgroup(2,0) ']'],'fontsize', ...
             8);
        mupbf=mupar; % to get the full fit parameter values
        l=length(mupout);
        ytop=0.22*dy;
        ybottom=-2*dy;
        dy=(ytop-ybottom)/l;
        %        ytop=xy(3)+ytop;
        ll=0;
        
% save present FFT in MU_DATA, "as is" (this LB and this range)
MU_DATA.FREQUENCY=f;
MU_DATA.FFTREAL=ar;
MU_DATA.FFTIMAG=ai;
MU_DATA.EFFT=ea;
%MU_DATA.FFTREAL(1:10)
        for k=1:l
            ytext=ytop-ll*dy;
            ll=ll+1;
            if mupout(k)~=0
                text(xy(2)+dtext,ytext,[cell2mat(parname(k)) ...
                                    '=' num2str(mupbf(k),3)],...
                     'BackgroundColor',[.9 .7 .7])
            else
                text(xy(2)+dtext,ytext,[cell2mat(parname(k)) ...
                                    '=' num2str(mupbf(k),3)])
            end
        end    
    end
    %     sliders
    % Text (Phase1) Value Slider
    % Text (Phase2) Value Slider
    Ytwidth=0.0395;
    Y=0.18-Ytwidth;
    Xtwidth=0.1;
    X=0.825;
    htext1=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Text',...
                 'String','\phi',...
                 'Horizontalalignment','Center',...
                 'Units','normalized',...
                 'Position',...
                 [X,Y,Xtwidth,Ytwidth]);
    hedit1=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Edit',...
                 'String',num2str(MU_FIG.FFTPHI),...
                 'Tag','Phi',...
                 'Callback',@Phase_CallBack,...
                 'Horizontalalignment','Left',...
                 'Units','normalized',...
                 'Position',...
                 [X+Xtwidth,Y,0.6*Xtwidth,Ytwidth],...
                 'Backgroundcolor',[1 1 1]);
    hslide1=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Slider',...
                 'Min',0,'Max',360,'SliderStep',[0.02 0.08],...
                 'Value',MU_FIG.FFTPHI,...
                 'Tag','SlidePhi',...
                 'Callback',@Phase_CallBack,...
                 'Horizontalalignment','Left',...
                 'Units','normalized',...
                 'Position',...
                 [X,Y-1.1*Ytwidth,1.7*Xtwidth,Ytwidth*0.9]);
    Y=Y-2.2*Ytwidth;
    htext2=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Text',...
                 'String','dPhi/dw',...
                 'Horizontalalignment','Left',...
                 'Units','normalized',...
                 'Position',...
                 [X,Y,Xtwidth,Ytwidth]);
    hedit2=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Edit',...
                 'String',num2str(MU_FIG.FFTDPHIDW),...
                 'Tag','dPhidw',...
                 'Callback',@Phase_CallBack,...
                 'Horizontalalignment','Left',...
                 'Units','normalized',...
                 'Position',...
                 [X+Xtwidth,Y,0.6*Xtwidth,Ytwidth],...
                 'Backgroundcolor',[1 1 1]);
    k=16*pi/FMAX; % max positive derivative
    hslide2=uicontrol('Parent',MU_FIG.FFT,...
                 'Style','Slider',...
                 'Min',-k,'Max',k,'SliderStep',[0.02 0.1],...
                 'Tag','SlidedPhidw',...
                 'Value',MU_FIG.FFTDPHIDW,...
                 'Callback',@Phase_CallBack,...
                 'Horizontalalignment','Left',...
                 'Units','normalized',...
                 'Position',...
                 [X,Y-1.1*Ytwidth,1.7*Xtwidth,Ytwidth*0.9]);
    %     SliderStep=[%Min %Max], Min, Max, Value
    %                 'Callback',@Plot_CallBack,...
    %
    set(MU_FIG.FFT,'Visible','on')
function Phase_CallBack(varargin)

global MU_FIG

hstrike=varargin{1};
h=guihandles;
tagmenu={'Phi' 'SlidePhi' 'dPhidw' 'SlidedPhidw'};

handles=[];

   for tag=tagmenu
       string=cell2mat(tag);
       handles=[handles getfield(h,string)];
   end

switch hstrike
  case handles(1) % Phi
    MU_FIG.FFTPHI=str2num(get(hstrike,'String'));
    mufftres; % regenerate figure with new phase 
  case handles(2) % SlidePhi
    phi=get(hstrike,'Value');
    MU_FIG.FFTPHI=phi;
    set(handles(1),'String',num2str(MU_FIG.FFTPHI));
    mufftres; % regenerate figure with new phase 
  case handles(3) % dPhidw
    MU_FIG.FFTDPHIDW=str2num(get(hstrike,'String'));
    mufftres; % regenerate figure with new phase
  case handles(4) % SlidedPhidw
    MU_FIG.FFTDPHIDW=get(hstrike,'Value');
    set(handles(3),'String',num2str(MU_FIG.FFTDPHIDW));
    mufftres; % regenerate figure with new phase
end
